@@ -1,4 +1,4 @@
-/* $Id: compile.h 1147 2006-02-20 15:45:49Z holger $
+/* $Id: compile.h 1270 2006-07-16 19:46:57Z holger $
  *
  * HoiChess/compile.h
  *
@@ -27,6 +27,8 @@
 
 #if defined(__GNUC__)
 # define COMPILER	"GCC " __VERSION__
+#elif defined(__INTEL_COMPILER)
+# define COMPILER	"ICC"
 #else
 # define COMPILER	"unknown"
 #endif
