@@ -1,4 +1,4 @@
-/* $Id: eval.cc 1171 2006-02-27 18:48:13Z holger $
+/* $Id: eval.cc 1256 2006-06-28 16:55:13Z holger $
  *
  * HoiChess/eval.cc
  *
@@ -126,8 +126,10 @@ unsigned int Evaluator::get_phase(const Board & board)
 }
 
 
-int Evaluator::eval(const Board & board, int alpha, int beta)
+int Evaluator::eval(const Board & board, int alpha, int beta, Color _myside)
 {
+	myside = _myside;
+	
 	int score;
 
 	/* TODO is_draw() still empty */
@@ -143,7 +145,7 @@ int Evaluator::eval(const Board & board, int alpha, int beta)
 				 board.get_material(xside));
 			
 	if (score >= beta + 300 || score <= alpha - 300) {
-		stat_evals_cut++;
+		STAT_INC(stat_evals_cut);
 		return score;
 	}
 	
@@ -162,7 +164,7 @@ int Evaluator::eval(const Board & board, int alpha, int beta)
 	}
 	
 	finish();
-	stat_evals_full++;
+	STAT_INC(stat_evals_full);
 
 #ifdef USE_EVALCACHE
 	if (evalcache) {
@@ -173,8 +175,9 @@ int Evaluator::eval(const Board & board, int alpha, int beta)
 	return score;
 }
 
-void Evaluator::print_eval(const Board & board, FILE * fp)
+void Evaluator::print_eval(const Board & board, Color _myside, FILE * fp)
 {
+	myside = _myside;
 	setup(&board);
 	
 	fprintf(fp, "material: %d/%d\n", 
@@ -187,6 +190,9 @@ void Evaluator::print_eval(const Board & board, FILE * fp)
 	fprintf(fp, "phase: %u\n", phase);
 
 	fprintf(fp, "draw: %s\n", is_draw(board) ? "yes" : "no");
+	fprintf(fp, "myside: %s\n",
+			(myside == WHITE ? "white"
+		 		: (myside == BLACK ? "black" : "none")));
 
 	fprintf(fp, "scoring plugins:\n");
 	for (int i=0; plugins[i].name != NULL; i++) {
@@ -242,8 +248,10 @@ void Evaluator::finish()
 
 void Evaluator::reset_statistics()
 {
+#ifdef COLLECT_STATISTICS
 	stat_evals_full = 0;
 	stat_evals_cut = 0;
+#endif
 
 	if (pawnhashtable) {
 		pawnhashtable->reset_statistics();
@@ -258,6 +266,7 @@ void Evaluator::reset_statistics()
 
 void Evaluator::print_statistics(FILE * fp) const
 {
+#ifdef COLLECT_STATISTICS
 	unsigned long evals_total = stat_evals_full + stat_evals_cut;
 	unsigned int perc_full = evals_total > 0
 		? 100 * stat_evals_full / evals_total
@@ -270,14 +279,15 @@ void Evaluator::print_statistics(FILE * fp) const
 			evals_total,
 			stat_evals_full, perc_full,
 			stat_evals_cut, perc_cut);
+#endif // COLLECT_STATISTICS
 
 	if (pawnhashtable) {
-		pawnhashtable->print_statistics();
+		pawnhashtable->print_statistics(fp);
 	}
 	
 #ifdef USE_EVALCACHE
 	if (evalcache) {
-		evalcache->print_statistics();
+		evalcache->print_statistics(fp);
 	}
 #endif
 }
@@ -311,7 +321,7 @@ const struct score_plugin Evaluator::plugins[] = {
 	{ "knights",	&Evaluator::score_knights	},
 	{ "bishops",	&Evaluator::score_bishops	},
 	{ "rooks",	&Evaluator::score_rooks		},
-	{ "queens",	&Evaluator::score_queens	},
+//	{ "queens",	&Evaluator::score_queens	},
 	{ "king",	&Evaluator::score_king		},
 	{ "devel",	&Evaluator::score_devel		},
 	{ "combo",	&Evaluator::score_combo		},
@@ -382,8 +392,8 @@ const int Evaluator::king_scores_endgame[] = {
  */
 
 #define EVAL_DOUBLEPAWNS	-5	/* TODO perhaps -50 ? */
-#define EVAL_EIGHTPAWNS		-10
-#define EVAL_PAWNRAMS		-10
+//#define EVAL_EIGHTPAWNS		-10
+//#define EVAL_PAWNRAMS		-10
 #define EVAL_ISOLATEDPAWN	-10
 #define EVAL_PASSEDPAWN(dist)	(25 + 80/(dist))
 #define EVAL_CONNECTEDPP	20
@@ -421,7 +431,7 @@ int Evaluator::score_pawns(Color side)
 
 #ifdef EVAL_EIGHTPAWNS
 	/* Penalize having eight pawns */
-	if (pawns.popcnt() == 8) {
+	if (side == myside  &&  pawns.popcnt() == 8) {
 		score += EVAL_EIGHTPAWNS;
 	}
 #endif
@@ -441,9 +451,11 @@ int Evaluator::score_pawns(Color side)
 
 #ifdef EVAL_PAWNRAMS
 		/* Pawn rams */
-		Square ram_sq = (side == WHITE) ? sq+8 : sq-8;
-		if (board->get_pawns(XSIDE(side)).testbit(ram_sq)) {
-			score += EVAL_PAWNRAMS;
+		if (side == myside) {
+			Square ram_sq = (side == WHITE) ? sq+8 : sq-8;
+			if (board->get_pawns(XSIDE(side)).testbit(ram_sq)) {
+				score += EVAL_PAWNRAMS;
+			}
 		}
 #endif
 
@@ -569,8 +581,8 @@ int Evaluator::score_bishops(Color side)
 
 #define EVAL_ROOKOPENFILE	10
 #define EVAL_ROOKHALFOPENFILE	5
-#define EVAL_ROOK7PAWNS7	20	
-#define EVAL_ROOK7KING8		50
+#define EVAL_ROOK7PAWNS7	20	// FIXME too high?
+#define EVAL_ROOK7KING8		50	// FIXME too high?
 #define EVAL_ROOKINFRONTPP	-15
 #define EVAL_ROOKBEHINDPP	25
 //#define EVAL_PINNEDROOK		-50
@@ -603,7 +615,7 @@ int Evaluator::score_rooks(Color side)
 
 #if defined(EVAL_ROOK7PAWNS7) || defined(EVAL_ROOK7KING8)
 		/* Rook on 7th rank and ...*/
-		if (RNK(sq) == rank7) {
+		if (phase != ENDGAME && RNK(sq) == rank7) {
 #ifdef EVAL_ROOK7PAWNS7
 			/* ... enemy pawns on 7th rank */
 			if (board->get_pawns(XSIDE(side)) 
@@ -661,6 +673,7 @@ int Evaluator::score_rooks(Color side)
  * Queen evaluation.
  */
 
+//#define EVAL_QUEENNOTPRESENT	-40
 //#define EVAL_QUEENNEARENEMYKING	5
 //#define EVAL_PINNEDQUEEN	-90
 
@@ -670,6 +683,14 @@ int Evaluator::score_queens(Color side)
 
 	Square sq;
 	Bitboard queens = board->get_queens(side);
+	
+#ifdef EVAL_QUEENNOTPRESENT
+	if (side == myside  &&  !queens) {
+		score += EVAL_QUEENNOTPRESENT;
+		return score;
+	}
+#endif
+	
 	while (queens) {
 		sq = queens.firstbit();
 		queens.clearbit(sq);
