@@ -1,4 +1,4 @@
-/* $Id: search.cc 1156 2006-02-26 08:48:58Z holger $
+/* $Id: search.cc 1259 2006-07-02 17:49:55Z holger $
  *
  * HoiChess/search.cc
  *
@@ -83,7 +83,7 @@ Search::~Search()
  *
  *****************************************************************************/
 
-void Search::start(Game _game, int _mode)
+void Search::start(Game _game, int _mode, Color _myside)
 {
 	DBG(2, "locking start_mutex");
 	start_mutex.lock();
@@ -92,6 +92,7 @@ void Search::start(Game _game, int _mode)
 	ASSERT(game == NULL);
 	game = &_game;
 	mode = _mode;
+	myside = _myside;
 
 	stop = false;
 	main();
@@ -108,10 +109,10 @@ void Search::start(const Board & board, const Clock & clock, int mode)
 {
 	Game game(board, clock, clock);
 	game.start();
-	start(game, mode);
+	start(game, mode, NO_COLOR);
 }
 
-void Search::start_thread(Game _game, int _mode)
+void Search::start_thread(Game _game, int _mode, Color _myside)
 {
 	DBG(2, "locking start_mutex");
 	start_mutex.lock();
@@ -122,6 +123,7 @@ void Search::start_thread(Game _game, int _mode)
 		ASSERT(game == NULL);
 		game = new Game(_game);
 		mode = _mode;
+		myside = _myside;
 		stop = false;
 		thread = new Thread(thread_main);
 		thread->start(this);
@@ -212,13 +214,13 @@ void Search::main()
 	tree.set_root(game->get_board());
 	rootdepth = 0;
 	const Board & rootboard = tree.get_rootboard();
-	int rooteval = evaluator->eval(rootboard, -INFTY, INFTY);
+	int rooteval = evaluator->eval(rootboard, -INFTY, INFTY, myside);
 	
 	if (verbose >= 2) {
 		printf("===================================================\n");
 		printf("Root node evaluation: %.2f\n",
 				(float) rooteval / 100);
-		evaluator->print_eval(rootboard);
+		evaluator->print_eval(rootboard, myside);
 		printf("---------------------------------------------------\n");
 		clock->print();
 		printf("===================================================\n");
@@ -410,7 +412,7 @@ int Search::search_root(unsigned int ply, int depth, int alpha, int beta)
 				print_result(depth, score, ' ');
 			}
 			if (score >= beta) {
-				stat_cut++;
+				STAT_INC(stat_cut);
 				break;
 			}
 		}
@@ -421,8 +423,10 @@ int Search::search_root(unsigned int ply, int depth, int alpha, int beta)
 	histtable[tree.get_board().get_side()]->add(node->get_best());
 #endif
 
+#ifdef COLLECT_STATISTICS
 	stat_moves_sum += moves;
 	stat_moves_cnt++;
+#endif
 
 	if (!shell->xboard) {
 		clear_line();
@@ -453,7 +457,7 @@ int Search::search(unsigned int ply, int depth, int alpha, int beta)
 	bool first = true;
 #endif
 	
-	stat_nodes++;
+	nodes++;
 	
 	/*
 	 * First look into the hash table if this
@@ -491,14 +495,15 @@ int Search::search(unsigned int ply, int depth, int alpha, int beta)
 
 #ifdef USE_NULLMOVE
 	/* Null-move forward pruning */
-	if (!node->in_check() && !(tree[ply-1]->get_played_move().is_null())
-			      && !(Evaluator::get_phase(tree.get_board())
-						== Evaluator::ENDGAME) ) {
+	bool null_ok = !(tree[ply-1]->get_played_move().is_null())
+		    && !(Evaluator::get_phase(tree.get_board())
+				   == Evaluator::ENDGAME);
+	if (!node->in_check() && null_ok) {
 		tree.make_move(Move::null());
-		score = -search(ply+1, depth-1-2, -beta, -beta+1);
+		score = -search(ply+1, depth-2-1, -beta, -beta+1);
 		tree.unmake_move();
 		if (score >= beta) {
-			stat_nullcut++;
+			STAT_INC(stat_nullcut);
 			return beta;
 		}
 	}
@@ -524,13 +529,52 @@ int Search::search(unsigned int ply, int depth, int alpha, int beta)
 	node->set_historytable(histtable[tree.get_board().get_side()]);
 #endif	
 
+	/*
+	 * Futility pruning, extended futility pruning, and razoring.
+	 */
+#ifdef USE_RAZORING
+	if (depth == 3  &&  (node->material_balance() + 900 <= alpha)) {
+		STAT_INC(stat_razcut);
+		depth--;
+	}
+#endif // USE_RAZORING
+
+#ifdef USE_FUTILITYPRUNING
+	bool fprune = false;
+	if (depth == 1  &&  (node->material_balance() + 300 <= alpha)) {
+		fprune = true;
+#ifdef USE_EXTENDED_FUTILITYPRUNING
+	} else if (depth == 2  &&  (node->material_balance() + 500 <= alpha)) {
+		STAT_INC(stat_xfutcut);
+		fprune = true;
+		depth--;
+#endif // USE_EXTENDED_FUTILITYPRUNING
+	}
+#endif // USE_FUTILITYPRUNING
+
+	
+	/*
+	 * Search all successor moves.
+	 */
 	for (Move mov = node->first(); mov; mov = node->next()) {
-		tree.make_move(mov);
+		Node * cnode = tree.make_move(mov);
 		if (!tree.get_board().is_legal()) {
 			tree.unmake_move();
 			continue;
 		}
 		moves++;
+
+#ifdef USE_FUTILITYPRUNING
+		/* Futility pruning */
+		if (fprune && !node->in_check() && !cnode->in_check()
+				&& !mov.is_capture()
+				&& !mov.is_enpassant()
+				&& !mov.is_promotion()) {
+			STAT_INC(stat_futcut);
+			tree.unmake_move();
+			continue;
+		}
+#endif
 		
 #ifdef USE_PVS
 		/* Search the current move. We use a standard
@@ -552,9 +596,9 @@ int Search::search(unsigned int ply, int depth, int alpha, int beta)
 
 		tree.unmake_move();
 
-		if (stat_nodes >= next_timecheck) {
+		if (nodes >= next_timecheck) {
 			check_time();
-			next_timecheck = stat_nodes + TIMECHECK_INTERVAL;
+			next_timecheck = nodes + TIMECHECK_INTERVAL;
 		}
 
 		if (stop) {
@@ -565,7 +609,7 @@ int Search::search(unsigned int ply, int depth, int alpha, int beta)
 			alpha = score;
 			node->set_best(mov);
 			if (score >= beta) {
-				stat_cut++;
+				STAT_INC(stat_cut);
 				break;
 			}
 		}
@@ -609,9 +653,11 @@ int Search::search(unsigned int ply, int depth, int alpha, int beta)
 		node->add_killer(node->get_best());
 	}
 #endif
-	
+
+#ifdef COLLECT_STATISTICS
 	stat_moves_sum += moves;
 	stat_moves_cnt++;
+#endif
 	
 	return alpha;
 }
@@ -624,7 +670,7 @@ int Search::quiescence_search(unsigned int ply, int alpha, int beta)
 	int score;
 	int moves = 0;
 	
-	stat_nodes_quiesce++;
+	nodes_quiesce++;
 
 	/*
 	 * First look into the hash table if this
@@ -663,7 +709,7 @@ int Search::quiescence_search(unsigned int ply, int alpha, int beta)
 	 * This score will be returned if none
 	 * of the moves is better than alpha.
 	 */
-	score = evaluator->eval(tree.get_board(), alpha, beta);
+	score = evaluator->eval(tree.get_board(), alpha, beta, myside);
 	if (score >= beta) {
 		/* TODO Is this really a good idea? */
 		return score;
@@ -695,7 +741,7 @@ int Search::quiescence_search(unsigned int ply, int alpha, int beta)
 			alpha = score;
 			node->set_best(mov);
 			if (score >= beta) {
-				stat_cut++;
+				STAT_INC(stat_cut);
 				break;
 			}
 		}
@@ -712,8 +758,10 @@ int Search::quiescence_search(unsigned int ply, int alpha, int beta)
 		hashtable->put(hashentry);
 	}
 
+#ifdef COLLECT_STATISTICS
 	stat_moves_sum_quiesce += moves;
 	stat_moves_cnt_quiesce++;
+#endif
 	
 	return alpha;
 }
