@@ -1,4 +1,4 @@
-/* $Id: shell_cmd.cc 1152 2006-02-21 20:29:14Z holger $
+/* $Id: shell_cmd.cc 1230 2006-06-17 16:08:53Z holger $
  *
  * HoiChess/shell_cmd.cc
  *
@@ -504,7 +504,14 @@ void Shell::cmd_show()
 			printf("\n");
 	} else if (param == "eval") {
 		Evaluator eval;
-		eval.print_eval(board);
+		printf("Symmetric evaluation:\n");
+		eval.print_eval(board, NO_COLOR);
+		printf("--------------------------------------------------\n");
+		printf("Evaluation if I would play white:\n");
+		eval.print_eval(board, WHITE);
+		printf("--------------------------------------------------\n");
+		printf("Evaluation if I would play black:\n");
+		eval.print_eval(board, BLACK);
 	} else if (param == "clocks") {
 		printf("[White]\n"); game->get_clock(WHITE).print();
 		printf("\n[Black]\n"); game->get_clock(BLACK).print();
